"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonApiSdkService = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const utils_1 = require("../utils");
class JsonApiSdkService {
    constructor(http, jsonApiUtilsService, jsonApiSdkConfig) {
        this.http = http;
        this.jsonApiUtilsService = jsonApiUtilsService;
        this.jsonApiSdkConfig = jsonApiSdkConfig;
    }
    getList(entity, params) {
        const query = this.jsonApiUtilsService.getQueryStringParams(params);
        return this.http
            .get(this.jsonApiUtilsService.getUrlForResource(entity.name), {
            params: query.toObject(),
        })
            .pipe((0, operators_1.map)((result) => {
            const resource = params
                ? this.jsonApiUtilsService.convertResponseData(result, params.include)
                : this.jsonApiUtilsService.convertResponseData(result);
            const { totalItems, pageSize, pageNumber } = Object.assign({
                totalItems: 0,
                pageNumber: 0,
                pageSize: 0,
            }, result.meta);
            return new utils_1.EntityArray(resource, {
                totalItems,
                pageNumber,
                pageSize,
            });
        }));
    }
    getAll(entity, params, push = true) {
        const request = this.getList(entity, params).pipe((0, rxjs_1.expand)((r) => {
            if (r.pageNumber * r.pageSize >= r.totalItems) {
                return rxjs_1.EMPTY;
            }
            const newParams = Object.assign(Object.assign({}, params), {
                page: {
                    number: r.pageNumber + 1,
                    size: r.pageSize,
                },
            });
            return this.getList(entity, newParams);
        }));
        if (!push) {
            return request.pipe((0, rxjs_1.reduce)((acum, item) => {
                if (!acum && !Array.isArray(acum)) {
                    acum = [];
                }
                acum.push(...item);
                return acum;
            }), (0, operators_1.map)((r) => new utils_1.EntityArray(r, {
                pageSize: r.length,
                pageNumber: 1,
                totalItems: r.length,
            })));
        }
        return request;
    }
    getOne(entity, id, params) {
        if (!id) {
            return (0, rxjs_1.throwError)(() => new Error('Id for resource is required'));
        }
        const query = this.jsonApiUtilsService.getQueryStringParams(params);
        return this.http
            .get(`${this.jsonApiUtilsService.getUrlForResource(entity.name)}/${id}`, {
            params: query.toObject(),
        })
            .pipe((0, operators_1.map)((result) => this.jsonApiUtilsService.convertResponseData(result, params === null || params === void 0 ? void 0 : params.include)));
    }
    postOne(entity) {
        const { attributes, relationships } = this.jsonApiUtilsService.generateBody(entity);
        const body = {
            data: Object.assign({ type: (0, utils_1.getTypeForReq)(entity.constructor.name), attributes }, (Object.keys(relationships).length > 0 ? { relationships } : {})),
        };
        return this.http
            .post(this.jsonApiUtilsService.getUrlForResource(entity.constructor.name), body)
            .pipe((0, operators_1.map)((r) => this.jsonApiUtilsService.convertResponseData(r)));
    }
    patchOne(entity) {
        const id = Reflect.get(entity, this.jsonApiSdkConfig.idKey);
        if (!id) {
            return (0, rxjs_1.throwError)(() => new Error('Resource params should be instance of resource with id params'));
        }
        const { attributes, relationships } = this.jsonApiUtilsService.generateBody(entity);
        const body = {
            data: {
                id: String(id),
                type: (0, utils_1.getTypeForReq)(entity.constructor.name),
            },
        };
        if (Object.keys(relationships).length > 0) {
            Reflect.set(body.data, 'relationships', relationships);
        }
        if (attributes && Object.keys(attributes).length > 0) {
            Reflect.set(body.data, 'attributes', attributes);
        }
        return this.http
            .patch(`${this.jsonApiUtilsService.getUrlForResource(entity.constructor.name)}/${id}`, body)
            .pipe((0, operators_1.map)((r) => this.jsonApiUtilsService.convertResponseData(r)));
    }
    deleteOne(entity) {
        const id = Reflect.get(entity, this.jsonApiSdkConfig.idKey);
        if (!id) {
            return (0, rxjs_1.throwError)(() => new Error('Resource params should be instance of resource with id params'));
        }
        return this.http.delete(`${this.jsonApiUtilsService.getUrlForResource(entity.constructor.name)}/${id}`);
    }
    getRelationships(entity, relationType) {
        const id = Reflect.get(entity, this.jsonApiSdkConfig.idKey);
        if (!id) {
            return (0, rxjs_1.throwError)(() => new Error('Resource params should be instance of resource with id params'));
        }
        return this.http
            .get(`${this.jsonApiUtilsService.getUrlForResource(entity.constructor.name)}/${id}/relationships/${String(relationType)}`)
            .pipe((0, operators_1.map)((result) => this.jsonApiUtilsService.getResultForRelation(result)));
    }
    patchRelationships(entity, relationType) {
        const id = Reflect.get(entity, this.jsonApiSdkConfig.idKey);
        if (!id) {
            return (0, rxjs_1.throwError)(() => new Error('Resource params should be instance of resource with id params'));
        }
        if (entity[relationType] === undefined) {
            return (0, rxjs_1.throwError)(() => new Error(`${relationType.toString()} should not be undefined in entity`));
        }
        const body = {
            data: this.jsonApiUtilsService.generateRelationshipsBody(entity[relationType]),
        };
        return this.http
            .patch(`${this.jsonApiUtilsService.getUrlForResource(entity.constructor.name)}/${id}/relationships/${String(relationType)}`, body)
            .pipe((0, operators_1.map)((result) => this.jsonApiUtilsService.getResultForRelation(result)));
    }
    postRelationships(entity, relationType) {
        const id = Reflect.get(entity, this.jsonApiSdkConfig.idKey);
        if (!id) {
            return (0, rxjs_1.throwError)(() => new Error('Resource params should be instance of resource with id params'));
        }
        if (entity[relationType] === undefined) {
            return (0, rxjs_1.throwError)(() => new Error(`${relationType.toString()} should not be undefined in entity`));
        }
        const body = {
            data: this.jsonApiUtilsService.generateRelationshipsBody(entity[relationType]),
        };
        return this.http
            .post(`${this.jsonApiUtilsService.getUrlForResource(entity.constructor.name)}/${id}/relationships/${String(relationType)}`, body)
            .pipe((0, operators_1.map)((result) => this.jsonApiUtilsService.getResultForRelation(result)));
    }
    deleteRelationships(entity, relationType) {
        const id = Reflect.get(entity, this.jsonApiSdkConfig.idKey);
        if (!id) {
            return (0, rxjs_1.throwError)(() => new Error('Resource params should be instance of resource with id params'));
        }
        if (entity[relationType] === undefined) {
            return (0, rxjs_1.throwError)(() => new Error(`${relationType.toString()} should not be undefined in entity`));
        }
        const body = {
            data: this.jsonApiUtilsService.generateRelationshipsBody(entity[relationType]),
        };
        return this.http
            .delete(`${this.jsonApiUtilsService.getUrlForResource(entity.constructor.name)}/${id}/relationships/${String(relationType)}`, body)
            .pipe((0, operators_1.map)(() => void 0));
    }
}
exports.JsonApiSdkService = JsonApiSdkService;
//# sourceMappingURL=json-api-sdk.service.js.map