"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonApiUtilsService = void 0;
const tslib_1 = require("tslib");
const json_api_nestjs_shared_1 = require("@yaser2us/json-api-nestjs-shared");
const change_case_commonjs_1 = require("change-case-commonjs");
const utils_1 = require("../utils");
const constants_1 = require("../constants");
class JsonApiUtilsService {
    constructor(jsonApiSdkConfig) {
        this.jsonApiSdkConfig = jsonApiSdkConfig;
    }
    getUrlForResource(resource) {
        const url = [(0, change_case_commonjs_1.kebabCase)(resource).toLocaleLowerCase()];
        if (this.jsonApiSdkConfig.apiPrefix) {
            url.unshift(this.jsonApiSdkConfig.apiPrefix);
        }
        return new URL(url.join('/'), this.jsonApiSdkConfig.apiHost).toString();
    }
    getQueryStringParams(params = {}) {
        let httpParams = this.getIncludeParam(params.include, new utils_1.HttpParams());
        httpParams = this.getSortParam(params.sort, httpParams);
        httpParams = this.getPageParam(params.page, httpParams);
        httpParams = this.getFieldParam(params.fields, httpParams);
        httpParams = this.getFilterParam(params.filter, httpParams);
        return httpParams;
    }
    getIncludeParam(include, httpParams) {
        if (include && Array.isArray(include) && include.length > 0) {
            return httpParams.set('include', include.join(','));
        }
        return httpParams;
    }
    getSortParam(sort, httpParams) {
        if (!sort || !(0, json_api_nestjs_shared_1.isObject)(sort) || Object.keys(sort).length === 0)
            return httpParams;
        const resultSortQuery = [];
        const { target } = sort, relation = tslib_1.__rest(sort, ["target"]);
        if (target) {
            for (const [key, val] of Object.entries(target)) {
                resultSortQuery.push(val === 'ASC' ? key : `-${key}`);
            }
        }
        for (const [relName, relProps] of Object.entries(relation)) {
            if (!(0, json_api_nestjs_shared_1.isObject)(relProps))
                continue;
            for (const [key, val] of Object.entries(relProps)) {
                resultSortQuery.push(val === 'ASC' ? `${relName}.${key}` : `-${relName}.${key}`);
            }
        }
        if (resultSortQuery.length > 0) {
            return httpParams.set('sort', resultSortQuery.join(','));
        }
        return httpParams;
    }
    getPageParam(page, httpParams) {
        if (!page || !(0, json_api_nestjs_shared_1.isObject)(page) || Object.keys(page).length === 0)
            return httpParams;
        const { number, size } = page;
        httpParams = httpParams.set(`page[number]`, `${number || 1}`);
        if (size) {
            return httpParams.set(`page[size]`, `${size}`);
        }
        return httpParams;
    }
    getFieldParam(field, httpParams) {
        if (!field || !(0, json_api_nestjs_shared_1.isObject)(field) || Object.keys(field).length === 0)
            return httpParams;
        const { target: targetProps = [] } = field, relation = tslib_1.__rest(field, ["target"]);
        if (targetProps.length > 0) {
            httpParams = httpParams.set('fields[target]', [...new Set(targetProps)].join(','));
        }
        for (const key in relation) {
            const fields = Reflect.get(relation, key);
            if (fields && fields.length > 0) {
                httpParams = httpParams.set(`fields[${key}]`, [...new Set(fields)].join(','));
            }
        }
        return httpParams;
    }
    getFilterParam(filter, httpParams) {
        if (!filter || !(0, json_api_nestjs_shared_1.isObject)(filter) || Object.keys(filter).length === 0)
            return httpParams;
        const { target } = filter, relation = tslib_1.__rest(filter, ["target"]);
        if (target) {
            for (const [key, val] of Object.entries(target)) {
                if (!(0, json_api_nestjs_shared_1.isObject)(val))
                    continue;
                for (const [operand, filter] of Object.entries(val)) {
                    httpParams = httpParams.set(`filter[${key}][${operand}]`, Array.isArray(filter)
                        ? filter
                            .map((f) => this.jsonApiSdkConfig.dateFields.includes(key) &&
                            f instanceof Date
                            ? f.toJSON()
                            : f.toString())
                            .join(',')
                        : filter === null
                            ? 'null'
                            : this.jsonApiSdkConfig.dateFields.includes(key) &&
                                filter instanceof Date
                                ? filter.toJSON()
                                : filter.toString());
                }
            }
        }
        for (const [relName, relProps] of Object.entries(relation)) {
            if (!(0, json_api_nestjs_shared_1.isObject)(relProps))
                continue;
            for (const [key, val] of Object.entries(relProps)) {
                if (!(0, json_api_nestjs_shared_1.isObject)(val))
                    continue;
                for (const [operand, filter] of Object.entries(val)) {
                    httpParams = httpParams.set(`filter[${relName}.${key}][${operand}]`, Array.isArray(filter) ? filter.join(',') : filter.toString());
                }
            }
        }
        return httpParams;
    }
    convertResponseData(body, includeEntity) {
        const { data, included = [] } = body;
        const isArray = Array.isArray(data);
        const arrayData = isArray ? data : [data];
        const result = [];
        for (const dataItem of arrayData) {
            const entityObject = Object.assign({ [this.jsonApiSdkConfig.idKey]: this.jsonApiSdkConfig.idIsNumber
                    ? parseInt(dataItem[this.jsonApiSdkConfig.idKey], 10)
                    : dataItem[this.jsonApiSdkConfig.idKey] }, Object.entries(dataItem.attributes || []).reduce((acum, [key, val]) => {
                acum[key] = this.jsonApiSdkConfig.dateFields.includes(key)
                    ? new Date(String(val))
                    : val;
                return acum;
            }, {}));
            const itemEntity = Object.assign(this.createEntityInstance(dataItem.type), entityObject);
            if (!includeEntity || includeEntity.length === 0) {
                result.push(itemEntity);
                continue;
            }
            for (const itemInclude of includeEntity) {
                if (!(dataItem.relationships && dataItem.relationships[itemInclude])) {
                    continue;
                }
                const relationship = dataItem.relationships[itemInclude];
                if (!relationship || !('data' in relationship) || !relationship.data) {
                    continue;
                }
                const relationshipData = relationship.data;
                if (Array.isArray(relationshipData)) {
                    itemEntity[itemInclude] = relationshipData.reduce((acum, item) => {
                        const result = this.findIncludeEntity(item, included);
                        if (result)
                            acum.push(result);
                        return acum;
                    }, []);
                }
                else {
                    const relation = this.findIncludeEntity(relationshipData, included);
                    if (relation)
                        itemEntity[itemInclude] = relation;
                }
            }
            result.push(itemEntity);
        }
        if (!isArray) {
            const firstItem = result[0];
            if (firstItem)
                return firstItem;
        }
        return result;
    }
    createEntityInstance(name) {
        return (0, json_api_nestjs_shared_1.createEntityInstance)(name);
    }
    findIncludeEntity(item, included) {
        const relatedIncluded = included.find((includedItem) => includedItem.type === item.type && includedItem.id === item.id);
        if (!relatedIncluded)
            return;
        const entityObject = Object.assign({ [this.jsonApiSdkConfig.idKey]: this.jsonApiSdkConfig.idIsNumber
                ? parseInt(relatedIncluded[this.jsonApiSdkConfig.idKey], 10)
                : relatedIncluded[this.jsonApiSdkConfig.idKey] }, Object.entries(relatedIncluded.attributes || []).reduce((acum, [key, val]) => {
            acum[key] = this.jsonApiSdkConfig.dateFields.includes(key)
                ? new Date(String(val))
                : val;
            return acum;
        }, {}));
        return Object.assign(this.createEntityInstance(item.type.toString()), entityObject);
    }
    generateBody(entity) {
        const attributes = Object.entries(entity)
            .filter(([key, val]) => {
            if (key === constants_1.ID_KEY)
                return false;
            const item = Array.isArray(val) ? val[0] : val;
            return !(0, utils_1.isRelation)(item);
        })
            .reduce((acum, [key, val]) => (Object.assign(Object.assign({}, acum), { [key]: val })), {});
        const relationships = json_api_nestjs_shared_1.ObjectTyped.entries(entity)
            .filter(([key, val]) => {
            if (key === constants_1.ID_KEY)
                return false;
            const item = Array.isArray(val) ? val[0] : val;
            return (0, utils_1.isRelation)(item);
        })
            .reduce((acum, [key, val]) => {
            let data;
            if (Array.isArray(val)) {
                data = val.map((i) => ({
                    type: (0, utils_1.getTypeForReq)(i.constructor.name),
                    id: i[constants_1.ID_KEY].toString(),
                }));
            }
            else {
                if (val[constants_1.ID_KEY] !== null) {
                    data = {
                        type: (0, utils_1.getTypeForReq)(val.constructor.name),
                        id: val[constants_1.ID_KEY].toString(),
                    };
                }
                else {
                    data = null;
                }
            }
            return Object.assign(Object.assign({}, acum), { [key]: { data } });
        }, {});
        return { attributes, relationships };
    }
    getResultForRelation(body) {
        if (Array.isArray(body.data)) {
            return body.data.map((i) => i[this.jsonApiSdkConfig.idKey]);
        }
        else {
            return body.data[this.jsonApiSdkConfig.idKey];
        }
    }
    generateRelationshipsBody(relationshipEntity) {
        const generateRelationObj = (relation) => ({
            id: String(Reflect.get(relation, this.jsonApiSdkConfig.idKey)),
            type: (0, utils_1.getTypeForReq)(relation.constructor.name),
        });
        return Array.isArray(relationshipEntity)
            ? relationshipEntity.map((item) => generateRelationObj(item))
            : generateRelationObj(relationshipEntity);
    }
}
exports.JsonApiUtilsService = JsonApiUtilsService;
//# sourceMappingURL=json-api-utils.service.js.map