import { createEntityInstance, isObject, ObjectTyped, } from '@yaser2us/json-api-nestjs-shared';
import { kebabCase } from 'change-case-commonjs';
import { getTypeForReq, HttpParams, isRelation } from '../utils';
import { ID_KEY } from '../constants';
export class JsonApiUtilsService {
    jsonApiSdkConfig;
    constructor(jsonApiSdkConfig) {
        this.jsonApiSdkConfig = jsonApiSdkConfig;
    }
    getUrlForResource(resource) {
        const url = [kebabCase(resource).toLocaleLowerCase()];
        if (this.jsonApiSdkConfig.apiPrefix) {
            url.unshift(this.jsonApiSdkConfig.apiPrefix);
        }
        return new URL(url.join('/'), this.jsonApiSdkConfig.apiHost).toString();
    }
    getQueryStringParams(params = {}) {
        let httpParams = this.getIncludeParam(params.include, new HttpParams());
        httpParams = this.getSortParam(params.sort, httpParams);
        httpParams = this.getPageParam(params.page, httpParams);
        httpParams = this.getFieldParam(params.fields, httpParams);
        httpParams = this.getFilterParam(params.filter, httpParams);
        return httpParams;
    }
    getIncludeParam(include, httpParams) {
        if (include && Array.isArray(include) && include.length > 0) {
            return httpParams.set('include', include.join(','));
        }
        return httpParams;
    }
    getSortParam(sort, httpParams) {
        if (!sort || !isObject(sort) || Object.keys(sort).length === 0)
            return httpParams;
        const resultSortQuery = [];
        const { target, ...relation } = sort;
        if (target) {
            for (const [key, val] of Object.entries(target)) {
                resultSortQuery.push(val === 'ASC' ? key : `-${key}`);
            }
        }
        for (const [relName, relProps] of Object.entries(relation)) {
            if (!isObject(relProps))
                continue;
            for (const [key, val] of Object.entries(relProps)) {
                resultSortQuery.push(val === 'ASC' ? `${relName}.${key}` : `-${relName}.${key}`);
            }
        }
        if (resultSortQuery.length > 0) {
            return httpParams.set('sort', resultSortQuery.join(','));
        }
        return httpParams;
    }
    getPageParam(page, httpParams) {
        if (!page || !isObject(page) || Object.keys(page).length === 0)
            return httpParams;
        const { number, size } = page;
        httpParams = httpParams.set(`page[number]`, `${number || 1}`);
        if (size) {
            return httpParams.set(`page[size]`, `${size}`);
        }
        return httpParams;
    }
    getFieldParam(field, httpParams) {
        if (!field || !isObject(field) || Object.keys(field).length === 0)
            return httpParams;
        const { target: targetProps = [], ...relation } = field;
        if (targetProps.length > 0) {
            httpParams = httpParams.set('fields[target]', [...new Set(targetProps)].join(','));
        }
        for (const key in relation) {
            const fields = Reflect.get(relation, key);
            if (fields && fields.length > 0) {
                httpParams = httpParams.set(`fields[${key}]`, [...new Set(fields)].join(','));
            }
        }
        return httpParams;
    }
    getFilterParam(filter, httpParams) {
        if (!filter || !isObject(filter) || Object.keys(filter).length === 0)
            return httpParams;
        const { target, ...relation } = filter;
        if (target) {
            for (const [key, val] of Object.entries(target)) {
                if (!isObject(val))
                    continue;
                for (const [operand, filter] of Object.entries(val)) {
                    httpParams = httpParams.set(`filter[${key}][${operand}]`, Array.isArray(filter)
                        ? filter
                            .map((f) => this.jsonApiSdkConfig.dateFields.includes(key) &&
                            f instanceof Date
                            ? f.toJSON()
                            : f.toString())
                            .join(',')
                        : filter === null
                            ? 'null'
                            : this.jsonApiSdkConfig.dateFields.includes(key) &&
                                filter instanceof Date
                                ? filter.toJSON()
                                : filter.toString());
                }
            }
        }
        for (const [relName, relProps] of Object.entries(relation)) {
            if (!isObject(relProps))
                continue;
            for (const [key, val] of Object.entries(relProps)) {
                if (!isObject(val))
                    continue;
                for (const [operand, filter] of Object.entries(val)) {
                    httpParams = httpParams.set(`filter[${relName}.${key}][${operand}]`, Array.isArray(filter) ? filter.join(',') : filter.toString());
                }
            }
        }
        return httpParams;
    }
    convertResponseData(body, includeEntity) {
        const { data, included = [] } = body;
        const isArray = Array.isArray(data);
        const arrayData = isArray ? data : [data];
        const result = [];
        for (const dataItem of arrayData) {
            const entityObject = {
                [this.jsonApiSdkConfig.idKey]: this.jsonApiSdkConfig.idIsNumber
                    ? parseInt(dataItem[this.jsonApiSdkConfig.idKey], 10)
                    : dataItem[this.jsonApiSdkConfig.idKey],
                ...Object.entries(dataItem.attributes || []).reduce((acum, [key, val]) => {
                    acum[key] = this.jsonApiSdkConfig.dateFields.includes(key)
                        ? new Date(String(val))
                        : val;
                    return acum;
                }, {}),
            };
            const itemEntity = Object.assign(this.createEntityInstance(dataItem.type), entityObject);
            if (!includeEntity || includeEntity.length === 0) {
                result.push(itemEntity);
                continue;
            }
            for (const itemInclude of includeEntity) {
                if (!(dataItem.relationships && dataItem.relationships[itemInclude])) {
                    continue;
                }
                const relationship = dataItem.relationships[itemInclude];
                if (!relationship || !('data' in relationship) || !relationship.data) {
                    continue;
                }
                const relationshipData = relationship.data;
                if (Array.isArray(relationshipData)) {
                    itemEntity[itemInclude] = relationshipData.reduce((acum, item) => {
                        const result = this.findIncludeEntity(item, included);
                        if (result)
                            acum.push(result);
                        return acum;
                    }, []);
                }
                else {
                    const relation = this.findIncludeEntity(relationshipData, included);
                    if (relation)
                        itemEntity[itemInclude] = relation;
                }
            }
            result.push(itemEntity);
        }
        if (!isArray) {
            const firstItem = result[0];
            if (firstItem)
                return firstItem;
        }
        return result;
    }
    createEntityInstance(name) {
        return createEntityInstance(name);
    }
    findIncludeEntity(item, included) {
        const relatedIncluded = included.find((includedItem) => includedItem.type === item.type && includedItem.id === item.id);
        if (!relatedIncluded)
            return;
        const entityObject = {
            [this.jsonApiSdkConfig.idKey]: this.jsonApiSdkConfig.idIsNumber
                ? parseInt(relatedIncluded[this.jsonApiSdkConfig.idKey], 10)
                : relatedIncluded[this.jsonApiSdkConfig.idKey],
            ...Object.entries(relatedIncluded.attributes || []).reduce((acum, [key, val]) => {
                acum[key] = this.jsonApiSdkConfig.dateFields.includes(key)
                    ? new Date(String(val))
                    : val;
                return acum;
            }, {}),
        };
        return Object.assign(this.createEntityInstance(item.type.toString()), entityObject);
    }
    generateBody(entity) {
        const attributes = Object.entries(entity)
            .filter(([key, val]) => {
            if (key === ID_KEY)
                return false;
            const item = Array.isArray(val) ? val[0] : val;
            return !isRelation(item);
        })
            .reduce((acum, [key, val]) => ({
            ...acum,
            [key]: val,
        }), {});
        const relationships = ObjectTyped.entries(entity)
            .filter(([key, val]) => {
            if (key === ID_KEY)
                return false;
            const item = Array.isArray(val) ? val[0] : val;
            return isRelation(item);
        })
            .reduce((acum, [key, val]) => {
            let data;
            if (Array.isArray(val)) {
                data = val.map((i) => ({
                    type: getTypeForReq(i.constructor.name),
                    id: i[ID_KEY].toString(),
                }));
            }
            else {
                if (val[ID_KEY] !== null) {
                    data = {
                        type: getTypeForReq(val.constructor.name),
                        id: val[ID_KEY].toString(),
                    };
                }
                else {
                    data = null;
                }
            }
            return {
                ...acum,
                [key]: { data },
            };
        }, {});
        return { attributes, relationships };
    }
    getResultForRelation(body) {
        if (Array.isArray(body.data)) {
            return body.data.map((i) => i[this.jsonApiSdkConfig.idKey]);
        }
        else {
            return body.data[this.jsonApiSdkConfig.idKey];
        }
    }
    generateRelationshipsBody(relationshipEntity) {
        const generateRelationObj = (relation) => ({
            id: String(Reflect.get(relation, this.jsonApiSdkConfig.idKey)),
            type: getTypeForReq(relation.constructor.name),
        });
        return Array.isArray(relationshipEntity)
            ? relationshipEntity.map((item) => generateRelationObj(item))
            : generateRelationObj(relationshipEntity);
    }
}
//# sourceMappingURL=json-api-utils.service.js.map